package com.chinaitop.agile.service.impl;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import javax.annotation.Resource;

import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.chinaitop.agile.feign.service.BasicEnumFeignService;
import com.chinaitop.agile.mapper.DataKcglFcbgzDefaultMapper;
import com.chinaitop.agile.mapper.DataKcglKcswDefaultMapper;
import com.chinaitop.agile.mapper.DataKcglXckcDefaultMapper;
import com.chinaitop.agile.model.DataKcglFcbgzDefault;
import com.chinaitop.agile.model.DataKcglKcswDefault;
import com.chinaitop.agile.model.DataKcglKcswDefaultExample;
import com.chinaitop.agile.model.DataKcglKcswDefaultExample.Criteria;
import com.chinaitop.agile.model.DataKcglXckcDefault;
import com.chinaitop.agile.model.DataKcswScedule;
import com.chinaitop.agile.service.KcswService;
import com.chinaitop.agile.utils.ParameterUtil;

@Service
public class KcswServiceImpl implements KcswService{

	@Resource
	private DataKcglKcswDefaultMapper dataKcglKcswDefaultMapper;
	@Resource
	private DataKcglXckcDefaultMapper dataKcglXckcDefaultMapper;
	@Resource
	private DataKcglFcbgzDefaultMapper dataKcglFcbgzDefaultMapper;
	@Resource
	private BasicEnumFeignService basicEnumFeignService;

	@Override
	public Map<String, Object> queryByExample(Integer orgId, String year, String lspz) throws Exception {
		/* 返回数据对象 */
		Map<String, Object> modelMap = new HashMap<String, Object>();

		/* 当前年份  */
		int this_year = Integer.parseInt(ParameterUtil.getSysDateYear());

		/* 封装年份  */
		String []year_num = new String[6];
		year_num[0] = this_year - 5 + "";
		year_num[1] = this_year - 4 + "";
		year_num[2] = this_year - 3 + "";
		year_num[3] = this_year - 2 + "";
		year_num[4] = this_year - 1 + "";
		year_num[5] = this_year + "";
		modelMap.put("years", year_num);
		
		int []month = new int[12];
		double []rk_count = new double[12];
		double []ck_count = new double[12];
		for (int i = 0; i < month.length; i++) {
			month[i] = i+1;
			rk_count[i] = 0d;
			ck_count[i] = 0d;
		}
		
		if (ParameterUtil.isnotnull(year) && ParameterUtil.isnotnull(lspz)) {
			double rksl = 0d; //入库数量
			double cksl = 0d; //出库数量
			DecimalFormat obj = new DecimalFormat(".0000");  // 保留4位小数
			String sumSlFormat = "";
			BigDecimal sumSl = null;
			
			Map<String, Object> param_map = new HashMap<String, Object>();
			param_map.put("year", year);
			param_map.put("lspz", lspz);
			param_map.put("orgId", orgId);
			List<Map<String, Object>> list = dataKcglFcbgzDefaultMapper.queryGroupByMonthAndPz(param_map);
			
			if (null != list && list.size() > 0) {
				for (int j = 0; j < list.size(); j++) {
					Map<String, Object> map = list.get(j);
					Double mon_d = Double.parseDouble(map.get("mon")+"");//月份
					int mon = ((int) Math.round(mon_d))-1; //月份所在的数组下标
					rksl = (double) map.get("rk_count");
					cksl = (double) map.get("ck_count");
					
					//处理入库数据，保留4位小数
					sumSlFormat = obj.format(rksl/1000);
					sumSl = new BigDecimal(sumSlFormat);
					rksl = Double.parseDouble(sumSl+"");
					rk_count[mon] = rksl;
					
					//处理出库数据，保留4位小数
					sumSlFormat = obj.format(cksl/1000);
					sumSl = new BigDecimal(sumSlFormat);
					cksl = Double.parseDouble(sumSl+"");
					ck_count[mon] = cksl;
				}
			}
		}

		modelMap.put("rk_count", rk_count);
		modelMap.put("ck_count", ck_count);
		modelMap.put("month", month);

		return modelMap;
	}

	public Map<String, Object> queryByDj(Integer orgId) {
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("orgId", orgId);
		List<DataKcswScedule> countByDj = dataKcglKcswDefaultMapper.countByDj(map);
		Map<String, Object> modelMap = new HashMap<String, Object>();
		StringBuilder yi=new StringBuilder();
		StringBuilder er=new StringBuilder();
		StringBuilder san=new StringBuilder();
		for (DataKcswScedule dataKcswScedule : countByDj) {
			String djmc = dataKcswScedule.getDjmc();
			if("3271".equals(djmc)){
				yi.append(dataKcswScedule.getSumCksl());
			}else if (djmc.equals("3272")) {
				er.append(dataKcswScedule.getSumCksl());
			}else if (djmc.equals("3273")) {
				san.append(dataKcswScedule.getSumCksl());
			}
		}
		modelMap.put("yi", yi);
		modelMap.put("er", er);
		modelMap.put("san", san);

		return modelMap;
	}

	public String queryByBl(Integer effectiveCapacity, DataKcglKcswDefaultExample example) {
		String sumCksl = dataKcglKcswDefaultMapper.countByBl(example);
		if(sumCksl!=null){
			BigDecimal sumCksls = new BigDecimal(sumCksl.toString());
			BigDecimal effectiveCapacitys = new BigDecimal(effectiveCapacity.toString());
			BigDecimal ss = new BigDecimal(1000);
			BigDecimal sumCksls2 = sumCksls.divide(ss);
			BigDecimal divide = sumCksls2.divide(effectiveCapacitys, 4, RoundingMode.CEILING);
			BigDecimal s = new BigDecimal(100);
			BigDecimal multiply = divide.multiply(s);
			float a = multiply.floatValue();
			String string = "";
			if (a > 100) {
				string = "100";
			} else {
				string = multiply.toString();
			}
			return string;
		}
		return "0";
	}

	@Override
	public List<DataKcglKcswDefault> findCondition(DataKcglKcswDefaultExample example) {

		List<DataKcglKcswDefault> selectByExample = dataKcglKcswDefaultMapper.selectByExample(example);
		return selectByExample;
	}

	@Override
	public List<DataKcglKcswDefault> getEmptyPolice(DataKcglKcswDefaultExample example) {
		return dataKcglKcswDefaultMapper.getEmptyPolice(example);
	}
	
	
	//获取仓房货位
	@Override
	public List<DataKcglKcswDefault> getHouseOrWareList(Map<String, Object> modelMap) {
		// TODO Auto-generated method stub
		return dataKcglKcswDefaultMapper.getHouseOrWareList(modelMap);
	}

	@Override
	public List<Map<String, Object>> selectCountByPz() {
		return dataKcglKcswDefaultMapper.countByPz();
	}

	@Override
	public List<Map<String, Object>> selectCountByXz() {
		return dataKcglKcswDefaultMapper.countByXz();
	}

	@Override
	public List<Map<String, Object>> selectCountByYear() {
		return dataKcglKcswDefaultMapper.countByYear();
	}

	@Override
	public List<Map<String, Object>> selectCountByCity() {
		return dataKcglKcswDefaultMapper.countByCity();
	}

	/*@Override
	public List<StorageQualitycheck> getCountBych(Map<String, Object> modelMap) {
		// TODO Auto-generated method stub
		return dataKcglKcswDefaultMapper.getCountBych(modelMap);
	}
*/
	@Override
	public List<DataKcglKcswDefault> getMassageBych(Map<String, Object> modelMap) {
		// TODO Auto-generated method stub
		return dataKcglKcswDefaultMapper.getMassageBych(modelMap);
	}

	@Override
	public List<DataKcglKcswDefault> queryLKNewData(Map<String, Object> map) {
		List<DataKcglKcswDefault> selectByExample = dataKcglKcswDefaultMapper.queryLKNewData(map);
		return selectByExample;
	}

	@Override
	public void updateKc(String userInfo, String profitlossSheet) {

		if (null == userInfo || null == profitlossSheet) {
			return;
		}
		JSONObject user = JSONObject.parseObject(userInfo);
		JSONObject sheet = JSONObject.parseObject(profitlossSheet);

		/* 删除对应仓房的库存实物数据 */
		DataKcglKcswDefaultExample example = new DataKcglKcswDefaultExample();
		Criteria criteria = example.createCriteria();
		criteria.andChEqualTo(Integer.parseInt(sheet.get("houseId").toString()));
		criteria.andHwhEqualTo(Integer.parseInt(sheet.get("warehouseId").toString()));
		DataKcglKcswDefault kcsw = dataKcglKcswDefaultMapper.selectByExample(example).get(0);
		dataKcglKcswDefaultMapper.deleteByPrimaryKey(kcsw.getDataid());

		/* 插入形成库存数据 */
		DataKcglXckcDefault dataKcglXckcDefault = new DataKcglXckcDefault();
		String uuid = UUID.randomUUID().toString().replace("-", "");
		dataKcglXckcDefault.setDataid(uuid);
		dataKcglXckcDefault.setUnitid(user.get("orgId").toString());
		dataKcglXckcDefault.setPeriodid(kcsw.getPeriodid());
		Byte roundid = 0;
		dataKcglXckcDefault.setRoundid(roundid);
		dataKcglXckcDefault.setModifydate(new Date());
		dataKcglXckcDefault.setBuilder(user.get("username").toString());
		dataKcglXckcDefault.setMxpz(sheet.get("grainKind").toString());
		dataKcglXckcDefault.setLkmc(user.get("orgName")==null?"":user.get("orgName").toString());
		dataKcglXckcDefault.setPz(sheet.get("grainKind").toString());
		dataKcglXckcDefault.setSjcf(Integer.parseInt(sheet.get("houseId").toString()));
		dataKcglXckcDefault.setDj(sheet.get("grainGrade").toString());
		dataKcglXckcDefault.setSjhw(Integer.parseInt(sheet.get("warehouseId").toString()));
		dataKcglXckcDefault.setScnf(sheet.get("productionYear").toString());
		dataKcglXckcDefault.setCazylx("溢出");
		double xzklhjz = Double.valueOf(sheet.get("incomeTonnage").toString());
		dataKcglXckcDefault.setXxklhjz(xzklhjz);
		dataKcglXckcDefault.setTzdh(0);
		dataKcglXckcDefault.setShoworder(0);
		dataKcglXckcDefault.setBizno("0");

		int count = dataKcglXckcDefaultMapper.insertSelective(dataKcglXckcDefault);

		if (count > 0) {
			/* 插入分仓保管账数据  */
			DataKcglFcbgzDefault dataKcglFcbgzDefault = new DataKcglFcbgzDefault();
			dataKcglFcbgzDefault.setDataid(uuid);
			dataKcglFcbgzDefault.setUnitid(user.get("orgId").toString());
			dataKcglFcbgzDefault.setRoundid(roundid);
			dataKcglFcbgzDefault.setPeriodid(kcsw.getPeriodid());
			dataKcglFcbgzDefault.setModifydate(new Date());
			dataKcglFcbgzDefault.setBuilder(dataKcglXckcDefault.getBuilder());
			dataKcglFcbgzDefault.setLkmc(dataKcglXckcDefault.getLkmc());
			dataKcglFcbgzDefault.setCh(kcsw.getCh());
			dataKcglFcbgzDefault.setHwh(kcsw.getHwh());
			dataKcglFcbgzDefault.setPz(dataKcglXckcDefault.getPz());
			dataKcglFcbgzDefault.setZcsl(xzklhjz);
			dataKcglFcbgzDefault.setKcsl(0d);
			dataKcglFcbgzDefault.setJzr(user.get("username").toString());
			dataKcglFcbgzDefault.setMxpz(dataKcglXckcDefault.getMxpz());
			dataKcglFcbgzDefault.setDj(dataKcglXckcDefault.getDj());
			dataKcglFcbgzDefault.setCjsj(new Date());
			dataKcglFcbgzDefault.setDirection(1);
			dataKcglFcbgzDefault.setScnf(dataKcglXckcDefault.getScnf());

			dataKcglFcbgzDefaultMapper.insertSelective(dataKcglFcbgzDefault);
		}
	}

	/**
	 * 根据orgId查询库存数量
	 */
	@Override
	public List<DataKcglKcswDefault> queryKcData(Integer orgId,Integer houseId,Integer warehouseId) {
		// TODO Auto-generated method stub
		DataKcglKcswDefaultExample example = new DataKcglKcswDefaultExample();
        Criteria createCriteria = example.createCriteria();
        if(orgId!=null){
        	createCriteria.andUnitidEqualTo(String.valueOf(orgId));
        }
        if(houseId!=null){
        	createCriteria.andChEqualTo(houseId);
        }
        if(warehouseId!=null){
        	createCriteria.andHwhEqualTo(warehouseId);
        }
		return dataKcglKcswDefaultMapper.selectByExample(example);
	}

	@Override
	public String countByBl(DataKcglKcswDefaultExample example) {
		return dataKcglKcswDefaultMapper.countByBl(example);
	}

	@Override
	public List<Map<String, Object>> getDateByHouseIds(Map<String, Object> map) {
		return dataKcglKcswDefaultMapper.getDateByHouseIds(map);
	}

	@Override
	public List<DataKcglKcswDefault> findListOrRcsj(DataKcglKcswDefault kcsw) {
		return dataKcglKcswDefaultMapper.findListOrRcsj(kcsw);
	}

	@Override
	public List<DataKcglKcswDefault> getList(Map<String, Object> map) {
		return dataKcglKcswDefaultMapper.getList(map);
	}

	@Override
	public String countByBl(Map<String, Object> map) {
		return dataKcglKcswDefaultMapper.queryKczl(map);
	}

	@Override
	public Map<String, Object> getFoodStatistics(DataKcglKcswDefaultExample example) {
		return dataKcglKcswDefaultMapper.getFoodStatistics(example);
	}

}
